@@ -15,93 +15,17 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/charset.h>
-#include <mobius/ref_counter.h>
+#include "charset.h"
 #include <mobius/exception_posix.inc>
-#include <unordered_map>
+#include <mobius/string_functions.h>
 #include <stdexcept>
 #include <memory>
 #include <iconv.h>
 
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Auxiliary class to convert from one charset to another
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-class Iconv
-{
-public:
-  Iconv (const Iconv&) = default;
-  Iconv (Iconv&&) = default;
-  Iconv& operator= (const Iconv&) = default;
-  Iconv& operator= (Iconv&&) = default;
-  Iconv (const std::string&, const std::string&);
-  ~Iconv ();
-  mobius::bytearray conv (const mobius::bytearray&) const;
-
-private:
-  iconv_t cd_ = (iconv_t) -1;
-  mobius::ref_counter counter_;
-};
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Constructor
-//! \param f_charset from charset
-//! \param t_charset to charset
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-Iconv::Iconv (const std::string& f_charset, const std::string& t_charset)
-{
-  cd_ = iconv_open (t_charset.c_str (), f_charset.c_str ());
-
-  if (cd_ == (iconv_t) -1)
-    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_POSIX);
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Destructor. Deallocate iconv structure
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-Iconv::~Iconv ()
-{
-  if (counter_.is_unique ())
-    iconv_close (cd_);
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Convert bytearray from one charset to another
-//! \param str Input bytearray
-//! \return Converted bytearray
-//! \todo Simplify output bytearray creation
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-mobius::bytearray
-Iconv::conv (const mobius::bytearray& str) const
-{
-  mobius::bytearray out;
-  size_t insize = str.size ();
-  size_t outsize = insize * 6;
-
-  char *inbuf = (char *) str.data ();
-  auto outbuf = std::make_unique<char[]> (outsize);
-  char *p_out = outbuf.get ();
-  
-  if (::iconv (cd_, &inbuf, &insize, &p_out, &outsize) != (size_t) -1)
-    {
-      // flush out partially converted input
-      iconv (cd_, nullptr, nullptr, &p_out, &outsize);
-
-      out = mobius::bytearray (reinterpret_cast <const uint8_t *> (outbuf.get ()), p_out - outbuf.get ());
-    }
-
-  return out;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief structure to hold charset conversion objects
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-using iconv_list_t = std::unordered_map <std::string, Iconv>;
-static iconv_list_t iconv_list;
-
 namespace mobius
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief convert a bytearray from "charset" to UTF-8
+//! \brief Convert bytearray from charset to UTF-8
 //! \param data bytearray
 //! \param charset charset
 //! \return string encoded as UTF-8
@@ -109,37 +33,136 @@ namespace mobius
 std::string
 conv_charset_to_utf8 (const mobius::bytearray& data, const std::string& charset)
 {
-  bytearray output = conv_charset (data, charset, "UTF-8");
+  const std::string u_charset = mobius::string::toupper (charset);
 
-  // strip trailings '\0'
-  auto size = output.size ();
-  while (size > 0 && output[size-1] == '\0')
-    --size;
+  if (u_charset == "UTF-8" || u_charset == "UTF8")
+    return data.to_string ();
 
-  return std::string (reinterpret_cast <const char *>(output.data ()), size);
+  else
+    {
+      bytearray output = conv_charset (data, u_charset, "UTF-8");
+
+      // strip trailings '\0'
+      auto size = output.size ();
+      while (size > 0 && output[size-1] == '\0')
+        --size;
+
+      return std::string (reinterpret_cast <const char *>(output.data ()), size);
+    }
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief convert a bytearray charset to UTF-8
+//! \brief Convert bytearray from charset to UTF-8, with partial convertion
+//! \param data bytearray
+//! \param charset charset
+//! \return string encoded as UTF-8 and remaining bytearray
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::pair <std::string, mobius::bytearray>
+conv_charset_to_utf8_partial (const mobius::bytearray& data, const std::string& charset)
+{
+  const std::string u_charset = mobius::string::toupper (charset);
+
+  if (u_charset == "UTF-8" || u_charset == "UTF8")
+    return std::make_pair (data.to_string (), mobius::bytearray ());
+
+  else
+    {
+      auto p = conv_charset_partial (data, u_charset, "UTF-8");
+      return std::make_pair (p.first.to_string (), p.second);
+    }
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Convert a bytearray from one charset to other charset
 //! \param data bytearray
 //! \param f_charset from charset
 //! \param t_charset to charset
 //! \return new bytearray
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::bytearray
-conv_charset (const mobius::bytearray& data, const std::string& f_charset, const std::string& t_charset)
+conv_charset (
+  const mobius::bytearray& data,
+  const std::string& f_charset,
+  const std::string& t_charset
+)
 {
-  std::string key = f_charset + '|' + t_charset;
+  // allocate descriptor 
+  const std::string uf_charset = mobius::string::toupper (f_charset);
+  const std::string ut_charset = mobius::string::toupper (t_charset);
+  iconv_t cd = iconv_open (ut_charset.c_str (), uf_charset.c_str ());
 
-  auto iter = iconv_list.find (key);
+  if (cd == (iconv_t) -1)
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_POSIX);
 
-  if (iter == iconv_list.end ())
+  // convert data
+  mobius::bytearray out;
+  size_t insize = data.size ();
+  size_t outsize = insize * 6;
+
+  char *inbuf = (char *) data.data ();
+  auto outbuf = std::make_unique<char[]> (outsize);
+  char *p_out = outbuf.get ();
+  
+  if (::iconv (cd, &inbuf, &insize, &p_out, &outsize) != (size_t) -1)
     {
-      auto p = iconv_list.emplace (key, Iconv (f_charset, t_charset));
-      iter = p.first;
-    }
+      // flush out partially converted input
+      iconv (cd, nullptr, nullptr, &p_out, &outsize);
 
-  return iter->second.conv (data);
+      out = mobius::bytearray (reinterpret_cast <const uint8_t *> (outbuf.get ()), p_out - outbuf.get ());
+    }
+  
+  // deallocate descriptor
+  iconv_close (cd);
+  
+  return out;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Convert bytearray from one charset to other, with partial convertion
+//! \param data bytearray
+//! \param f_charset from charset
+//! \param t_charset to charset
+//! \return new bytearray and remaining bytearray
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::pair <mobius::bytearray, mobius::bytearray>
+conv_charset_partial (
+  const mobius::bytearray& data,
+  const std::string& f_charset,
+  const std::string& t_charset
+)
+{
+  // allocate descriptor 
+  const std::string uf_charset = mobius::string::toupper (f_charset);
+  const std::string ut_charset = mobius::string::toupper (t_charset);
+  iconv_t cd = iconv_open (ut_charset.c_str (), uf_charset.c_str ());
+
+  if (cd == (iconv_t) -1)
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_POSIX);
+
+  // convert data
+  mobius::bytearray out;
+  mobius::bytearray rest;
+  size_t insize = data.size ();
+  size_t outsize = insize * 6;
+
+  char *inbuf = (char *) data.data ();
+  auto outbuf = std::make_unique <char[]> (outsize);
+  char *p_out = outbuf.get ();
+  
+  size_t rc = ::iconv (cd, &inbuf, &insize, &p_out, &outsize);
+
+  if (rc != (size_t) -1 || errno == EINVAL)
+    {
+      out = mobius::bytearray (reinterpret_cast <const uint8_t *> (outbuf.get ()), p_out - outbuf.get ());
+
+      if (inbuf < (char *) data.end ())
+        rest = mobius::bytearray (reinterpret_cast <const uint8_t *> (inbuf), (char *) data.end () - inbuf);
+    }
+  
+  // deallocate descriptor
+  iconv_close (cd);
+  
+  return std::make_pair (out, rest);
 }
 
 } // namespace mobius
