@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,67 +15,53 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/datetime/time.h>
+#include <mobius/datetime/time.h>  // must be full qualified to avoid system's time.h
 
 namespace mobius
 {
 namespace datetime
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief constructor
-//! \param h hour
-//! \param m minute (0 - 59)
-//! \param s second (0 - 59)
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-time::time (resolution_type h, resolution_type m, resolution_type s)
-  : hour_ (h),
-    minute_ (m),
-    second_ (s),
-    is_valid_ (true)
-{
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief construct time from day second number
+//! \brief Construct time from day second number
 //! \param s day second
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-time::time (day_second_t s)
+time::time (day_second_type s) noexcept
 {
   from_day_seconds (s);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief set time from second
+//! \brief Set time from second
 //! \param s day second
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 void
-time::from_day_seconds (day_second_t s)
+time::from_day_seconds (day_second_type s) noexcept
 {
   second_ = s % 60;
   s = s / 60;
   minute_ = s % 60;
   hour_ = s / 60;
-  is_valid_ = true;
+  is_null_ = false;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief return number of second of the day
+//! \brief Get total seconds
 //! \return number of second
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-day_second_t
-time::to_day_seconds () const
+day_second_type
+time::to_day_seconds () const noexcept
 {
   return second_ + minute_ * 60 + hour_ * 3600;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief compare two time objects
+//! \brief Compare two time objects
 //! \param t1 time object
 //! \param t2 time object
 //! \return true if the two time objects are equal
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
-operator== (const time& t1, const time& t2)
+operator== (const time& t1, const time& t2) noexcept
 {
   return bool (t1) == bool (t2) &&
          t1.get_hour () == t2.get_hour () &&
@@ -84,14 +70,15 @@ operator== (const time& t1, const time& t2)
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief write datetime representation to std::ostream
-//! \param stream ostream reference
-//! \param dt datetime object
-//! \return reference to ostream
+//! \brief Convert time to string
+//! \param t time object
+//! \return string
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-std::ostream&
-operator<< (std::ostream& stream, const time& t)
+std::string
+to_string (const time& t)
 {
+  std::string str;
+
   if (t)
     {
       char buffer[64] = {0};
@@ -101,9 +88,23 @@ operator<< (std::ostream& stream, const time& t)
                t.get_minute (),
                t.get_second ());
 
-      stream << buffer;
+      str = buffer;
     }
 
+  return str;
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Write datetime representation to std::ostream
+//! \param stream ostream reference
+//! \param dt datetime object
+//! \return reference to ostream
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+std::ostream&
+operator<< (std::ostream& stream, const time& t)
+{
+  stream << to_string (t);
+
   return stream;
 }
 
