@@ -149,14 +149,27 @@ tp_f_get_authority (io_uri_o *self, PyObject *)
 //! \return Path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
-tp_f_get_path (io_uri_o *self, PyObject *)
+tp_f_get_path (io_uri_o *self, PyObject *args)
 {
+  // parse input args
+  std::string arg_encoding;
+
+  try
+    {
+      arg_encoding = mobius::py::get_arg_as_std_string (args, 0, std::string ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = mobius::py::pystring_from_std_string (self->obj->get_path ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_path (arg_encoding));
     }
   catch (const std::exception& e)
     {
@@ -174,14 +187,27 @@ tp_f_get_path (io_uri_o *self, PyObject *)
 //! \return Query
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
-tp_f_get_query (io_uri_o *self, PyObject *)
+tp_f_get_query (io_uri_o *self, PyObject *args)
 {
+  // parse input args
+  std::string arg_encoding;
+
+  try
+    {
+      arg_encoding = mobius::py::get_arg_as_std_string (args, 0, std::string ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = mobius::py::pystring_from_std_string (self->obj->get_query ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_query (arg_encoding));
     }
   catch (const std::exception& e)
     {
@@ -199,14 +225,27 @@ tp_f_get_query (io_uri_o *self, PyObject *)
 //! \return Fragment
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyObject *
-tp_f_get_fragment (io_uri_o *self, PyObject *)
+tp_f_get_fragment (io_uri_o *self, PyObject *args)
 {
+  // parse input args
+  std::string arg_encoding;
+
+  try
+    {
+      arg_encoding = mobius::py::get_arg_as_std_string (args, 0, std::string ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = mobius::py::pystring_from_std_string (self->obj->get_fragment ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_fragment (arg_encoding));
     }
   catch (const std::exception& e)
     {
