@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+import traceback
 import mobius
 import pymobius.registry.main
 
@@ -48,8 +49,8 @@ class Ant (object):
 
     try:
       self.__retrieve_ie_passwords ()
-    except Exception, e:
-      mobius.core.logf ('WRN ant.turing: %s' % str (e))
+    except Exception as e:
+      mobius.core.logf ('WRN ' + str (e) + ' ' + traceback.format_exc ())
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Event <i>on_export_data</i>: Export data to mediator ant
@@ -63,8 +64,8 @@ class Ant (object):
   # @param ant Mediator ant object
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   def on_stop (self, ant):
-    mobius.core.logf ('INF ant.turing: %d/%d Internet Explorer password(s) decrypted' % (self.__decrypted_count, self.__total_count))
-    
+    mobius.core.logf ('INF %d/%d Internet Explorer password(s) decrypted' % (self.__decrypted_count, self.__total_count))
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Event <i>on_key</i>: Process key object
   # @param ant Mediator ant object
@@ -99,8 +100,8 @@ class Ant (object):
         self.__total_count -= 1
 
     else:
-      mobius.core.logf ('WRN ant.turing: IE DPAPI blob not decrypted')
-      
+      mobius.core.logf ('WRN IE DPAPI blob not decrypted')
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve IE passwords
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -110,8 +111,8 @@ class Ant (object):
 
       for registry in ant.get_data ():
         self.__retrieve_ie_passwords_from_registry (registry)
-    except Exception, e:
-      mobius.core.logf ('WRN ant.turing: %s' % str(e))
+    except Exception as e:
+      mobius.core.logf ('WRN ' + str (e) + ' ' + traceback.format_exc ())
 
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve IE password from registry
@@ -134,8 +135,8 @@ class Ant (object):
         self.__secrets.append (secret)
 
       else:
-        mobius.core.logf ('WRN ant.turing: IE entropy not found: %s' % value.name)
-              
+        mobius.core.logf ('WRN IE entropy not found: %s' % value.name)
+
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   # @brief Retrieve IE password from decrypted data
   # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -148,7 +149,7 @@ class Ant (object):
 
     index_offset = header_size
     data_offset = header_size + index_size
-    
+
     signature = decoder.get_string_by_size (4)
     if signature != 'WICK':
       return None
@@ -157,17 +158,17 @@ class Ant (object):
     count = decoder.get_uint32_le ()
     values = []
     decoder.seek (pos + index_offset)
-    
-    for i in xrange (count):
+
+    for i in range (count):
       offset = decoder.get_uint32_le ()
       timestamp = decoder.get_nt_datetime ()
       size = decoder.get_uint32_le ()
-      value = unicode (data[data_offset + offset: data_offset + offset + size * 2], 'utf-16le')
+      value = data[data_offset + offset: data_offset + offset + size * 2].decode ('utf-16le')
       values.append (value)
 
     passwords = []
 
-    for i in xrange (0, len (values), 2):
+    for i in range (0, len (values), 2):
       username = values[i]
       password = values[i+1]
 
@@ -183,5 +184,5 @@ class Ant (object):
         p.metadata.append (('Application', 'Internet Explorer'))
 
         passwords.append (p)
-    
+
     return passwords
