@@ -15,16 +15,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+import sqlite3
+import os
+import traceback
 import mobius
 import pymobius
 import pymobius.app.chromium
-import sqlite3
-import binascii
-import os
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Load data from Cookies file
-# Tested versions: 5, 7-12
+# Tested versions: 4-5, 7-16
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 def decode (f):
   cookies = []
@@ -32,8 +32,8 @@ def decode (f):
   try:
     cookies = decode_file (f)
 
-  except Exception, e:
-    mobius.core.logf ('WRN app.chromium.decoder_cookies: %s' % str (e))
+  except Exception as e:
+    mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
 
   return cookies
 
@@ -43,7 +43,7 @@ def decode (f):
 def decode_file (f):
   cookies = []
 
-  # check if file is valid  
+  # check if file is valid
   if not f or f.is_reallocated ():
     return cookies
 
@@ -59,16 +59,16 @@ def decode_file (f):
   version = pymobius.app.chromium.get_meta_version (db)
 
   if version:
-    mobius.core.logf ('INF app.chromium: Cookies.version = %s' % version)
+    mobius.core.logf ('INF Cookies.version = %s' % version)
 
-    if version <= 5:
-      cookies = decode_cookies_5 (db, f)
+    if version < 7:
+      cookies = decode_cookies_4 (db, f)
 
     else:
       cookies = decode_cookies_7 (db, f)
 
   else:
-    mobius.core.logf ('WRN app.chromium.decoder_cookies: unknown Cookies.version')
+    mobius.core.logf ('WRN Unknown Cookies.version')
 
   db.close ()
 
@@ -80,9 +80,9 @@ def decode_file (f):
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Retrieve data from cookies table
-# Tested versions: 5
+# Tested versions: 4-5
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-def decode_cookies_5 (db, f):
+def decode_cookies_4 (db, f):
   cookies = []
   cursor = db.cursor ()
 
@@ -92,7 +92,8 @@ def decode_cookies_5 (db, f):
               name,
               value,
               last_access_utc,
-              creation_utc
+              creation_utc,
+              expires_utc
          FROM cookies'''
 
   for row in cursor.execute (stmt):
@@ -104,16 +105,17 @@ def decode_cookies_5 (db, f):
     entry.is_encrypted = False
     entry.last_access_time = mobius.datetime.new_datetime_from_nt_timestamp (row[4] * 10)
     entry.creation_time = mobius.datetime.new_datetime_from_nt_timestamp (row[5] * 10)
+    entry.expiration_time = mobius.datetime.new_datetime_from_nt_timestamp (row[6] * 10)
     entry.evidence_path = f.path
     entry.is_deleted = f.is_deleted ()
 
     cookies.append (entry)
-    
+
   return cookies
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Retrieve data from cookies table
-# Tested versions: 7-15
+# Tested versions: 7-16
 # @todo Decrypt encrypted_value using DPAPI
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 def decode_cookies_7 (db, f):
@@ -126,21 +128,26 @@ def decode_cookies_7 (db, f):
               name,
               encrypted_value,
               last_access_utc,
-              creation_utc
+              creation_utc,
+              expires_utc
          FROM cookies'''
 
   for row in cursor.execute (stmt):
-    entry = pymobius.Data ()
-    entry.id = row[0]
-    entry.domain = (row[1] or '').lstrip ('.')
-    entry.name = row[2]
-    entry.value = binascii.hexlify (row[3])
-    entry.is_encrypted = True
-    entry.last_access_time = mobius.datetime.new_datetime_from_nt_timestamp (row[4] * 10)
-    entry.creation_time = mobius.datetime.new_datetime_from_nt_timestamp (row[5] * 10)
-    entry.evidence_path = f.path
-    entry.is_deleted = f.is_deleted ()
+    try:
+      entry = pymobius.Data ()
+      entry.id = row[0]
+      entry.domain = (row[1] or '').lstrip ('.')
+      entry.name = row[2]
+      entry.value = row[3]
+      entry.is_encrypted = True
+      entry.last_access_time = mobius.datetime.new_datetime_from_nt_timestamp (row[4] * 10)
+      entry.creation_time = mobius.datetime.new_datetime_from_nt_timestamp (row[5] * 10)
+      entry.expiration_time = mobius.datetime.new_datetime_from_nt_timestamp (row[6] * 10)
+      entry.evidence_path = f.path
+      entry.is_deleted = f.is_deleted ()
+    except Exception as e:
+      mobius.core.logf ('WRN %s %s' % (str (e), traceback.format_exc ()))
 
     cookies.append (entry)
-    
+
   return cookies
